#include <qregexp.h>
#include <qtextstream.h>

#include "qhttpservletrequest.h"

QHttpServletRequest::QHttpServletRequest()
   :  QServletRequest(), hasHdrs(false)
{
}

QHttpServletRequest::~QHttpServletRequest()
{
}

void QHttpServletRequest::addHeader( const QString &name, const QString &val )
{
   hdrNames.append( name );
   headers[name] = val;
}

QDateTime QHttpServletRequest::dateHeader( const QString &name ) const
{
   return QDateTime::fromString( header(name) );
}

int QHttpServletRequest::intHeader( const QString &name, bool *ok ) const
{
   QString s( header(name) );
   return s.toInt( ok );
}

//
// Request parsing methods
//

bool QHttpServletRequest::readRequest( QTextStream *in )
{
   bool ok = readMethod( in->readLine() );
   if ( !ok )
      return false;

   if ( hasHdrs ) {
      QString line = in->readLine();
      while ( ok && !line.isEmpty() ) {
         ok = readHeader( line );
         if ( ok )
            line = in->readLine();
      }
   }

   return ok;
}

bool QHttpServletRequest::readMethod( const QString &line )
{
//  rmDebug(line);
   // Extract method and uri
   QRegExp re( "^(\\w+)\\s+(\\S+)" );
   int pos = line.find( re );
   if ( pos < 0 )
      return false;

   meth = re.cap(1);
   uri = re.cap(2);
   url = uri;

   // Extract HTTP version
   QRegExp vre( "HTTP/(\\d\\.\\d)", false );
   pos = line.find( vre, pos );

   // If HTTP version 1.0 or higher
   if ( pos >= 0 ) {
      hasHdrs = true;
      ver = vre.cap(1);
   }
   else
      ver = "0.9";
      
   return true;
}

bool QHttpServletRequest::readHeader( const QString &hdr )
{
   if ( !hasHdrs ) {
//      rmWarning("QHttpServletRequest received unexpected readHeader() call");
      return false;
   }

   // Read and store the headers
   QRegExp re( "^([\\w-_]+):\\s+(.*)" );
   int pos = re.search( hdr );
   if ( pos < 0 )
      return false;

   addHeader( re.cap(1), re.cap(2) );

   return true;
}

bool QHttpServletRequest::readHeaders( const QStringList &lines )
{
   if ( !hasHdrs ) {
//      rmWarning("QHttpServletRequest received unexpected readHeaders() call");
      return false;
   }

   // Read and store the headers
   for ( QStringList::ConstIterator it = lines.begin(); it != lines.end(); ++it )
      readHeader( *it );

   // Process special headers
   if ( headers.contains("ContentLength") ) {
      bool ok;
      long len = headers["ContentLength"].toLong( &ok );
      if ( !ok )
	 return false;

      setContentLength( len );
   }

   if ( headers.contains("ContentType") )
      setContentType( headers["ContentType"] );
 
//#warning "Content encoding not supported"

   return true;
}

QString QHttpServletRequest::dump()
{
   QString s;
   QTextOStream out( &s );

   out << "Method: " << meth << endl;
   out << "URI: " << uri << endl;
   out << "Version: " << ver << endl;

   out << "URL: " << url << endl;
   out << "Path Info: " << pathInfo() << endl;
   out << "Query String: " << queryString() << endl;

   out << "Server Name: " << serverName() << endl;
   out << "Server Port: " << serverPort() << endl;
   out << "Remote Address: " << remoteAddress() << endl;
   out << "Remote Host: " << remoteHost() << endl;

   out << "Request Headers: " << (hasHdrs ? "supported" : "not supported") << endl;
   out << "Headers:" << endl;
   for ( QStringList::ConstIterator it = hdrNames.begin(); it != hdrNames.end(); ++it )
      out << "         '" << *it << "' => '" << headers[*it] << "'" << endl;

   return s;
}


// Local Variables:
// c-basic-offset: 3
// End:

